#include "main.h"

//Resfile res;
extern Resfile res;
Parser parsLine;
Parser parsSpace;



bool Sprite::loadGfx(const char *name, Sprite *sprite)
	{
	char *resGfx=res.load(name);
	if(resGfx==NULL)
		{
		printf("FAIL: Sprite.loadGfx: Nie udalo sie wczytac grafiki (%s)\n", name);
		return 0;
		}
	SDL_Surface *tmp=SDL_LoadBMP_RW(SDL_RWFromMem(resGfx, res.getLastSize()), 1);	// Ustawianie gfx do sprite

	sprite->sprite=SDL_DisplayFormatAlpha(tmp);											// 'Optymalizacja'
	SDL_FreeSurface(tmp);														// Zwalnianie miejsca
	delete [] resGfx;
	return 1;
	}

bool Sprite::loadAlpha(const char *name, Sprite *sprite)
	{
	char *smt=res.load("", name);
	Uint8 *pix=(Uint8 *)sprite->sprite->pixels+3;		// Do zmiany jesli stwierdzisz, ze 32bitowy surface to zly pomysl	// olac, przerobione na Uint8... chociaz nie, i tak trza bedzie przerobic
	if(smt==NULL)
		{
		//SDL_SetColorKey(sprite->sprite, SDL_SRCCOLORKEY, SDL_MapRGB(screen->format, 255, 0, 0));
		// #warning Do odkomentowania
		pix-=3;
		int size=sprite->sprite->w*sprite->sprite->h*4;			// Jw., tez do ewentualnej zmiany
		for(int j=0; j<size; j+=4)
			if(pix[j]==255 && pix[j+1]!=255 && pix[j+2]==255)
				pix[j+3]=0;
		}
	else
		{
		//printf("OK  : Sprite.load: Znaleziono plik przezroczystosci (%s)\n", name);
		if(res.getLastSize()!=(unsigned int)sprite->sprite->w*sprite->sprite->h)
			{
			printf("FAIL: Sprite.loadAlpha: Niezgodne wielkosci obrazow (%s)\n", name);
			delete [] smt;
			return 0;
			}
		for(unsigned int j=0; j<res.getLastSize(); j++)
			pix[j*4]=smt[j];
		}
	delete [] smt;
	return 1;
	}

bool Sprite::loadMask(Sprite *sprite)
	{
	Uint8 *pix=(Uint8 *)sprite->sprite->pixels+3;
	try
		{
		sprite->mask=new bool*[sprite->sprite->h];
		for(int y=0; y<sprite->sprite->h; y++)
			{
			sprite->mask[y]=new bool[sprite->sprite->w];
			for(int x=0; x<sprite->sprite->w; x++)
				sprite->mask[y][x]=pix[x+y*sprite->sprite->w] >> 24;
				//sprite->mask[y][x]=*(smt++);
			//memcpy(sprite->mask, smtMask, sprite->sprite->w);			// alternatywne metody, porownywalnie szybkie
			//smtMask+=sprite->sprite->w;
			}
		}
	catch(bad_alloc &)
		{
		printf("FAIL: Sprite.loadMask: Nie udalo sie zajac miejsca na maske\n");
		return 0;
		}
	return 1;
	}

bool Sprite::loadAnims(const char *name, Sprite *sprite)
	{
	char *tmp=res.load(name);
	if(tmp==NULL)
		{
		printf("FAIL: Sprite.loadAnim: Nie udalo sie wczytac informacji o animacjach (%s)\n", name);
		return 0;
		}
	parsLine=tmp;
	for(int j=0; j<parsLine.count(); j++)
		{
		parsSpace=parsLine[j];
		if(parsSpace.count()<7)
			{
			printf("FAIL: Sprite.loadAnim: Brak wszystkich wymaganych informacji o animacji (%s)\n", name);
			continue;
			}
		else if(parsSpace[0]=="#")
			continue;
		sprite->anims.push_back(Anim());					// wektor struktur -> push_back(konstruktor)
		sprite->anims.back().frames=parsSpace.val(0);
		sprite->anims.back().x0=parsSpace.val(1);
		sprite->anims.back().y0=parsSpace.val(2);
		sprite->anims.back().w=parsSpace.val(3);
		sprite->anims.back().h=parsSpace.val(4);
		sprite->anims.back().spotx=parsSpace.val(5);
		sprite->anims.back().spoty=parsSpace.val(6);		// Fragment przekopiowany z platformera, wersja 1
		if(parsSpace.count()>7)
			{
			sprite->anims.back().speed=parsSpace.val(7);
			if(parsSpace.count()>8)
				sprite->anims.back().returnto=parsSpace.val(8);
			}
		else
			{
			sprite->anims.back().speed=100;
			sprite->anims.back().returnto=0;
			}
		}
	//printf("INFO: Sprite.loadAnim: Ilosc animacji: %d\n", parsLine.count());
	return 1;
	}

void Sprite::load(string filename, map <string, Sprite*> &gfx)
	{
	parsLine="";
	parsSpace="";
	parsLine.setDelimiter("\n\r");
	parsSpace.setDelimiter(" \t");
/********** Czyszczenie **********/
	if(gfx.size()>0)
		clear(gfx);
/********** Wczytywanie grafik **********/
	int count=res.getFileCount(filename);
	for(int i=0; i<count; i++)
		{
		const char *name=res.getFileName(i).c_str();		// Nie kasowac, wskaznik na nazwe z naglowka resfile... chyba
		if(name==NULL)
			{
			printf("FAIL: Sprite.load: Nie znaleziono pliku %d\n", i);		// Wtf is that i dont even
			continue;
			}
		if(strcmp(name+strlen(name)-3, "bmp")!=0)
			continue;
		//char *nameGfx=NULL;
		char *nameSmt=NULL;
		char *nameAnim=NULL;
		try
			{
			//nameGfx=new char[strlen(name)];
			nameSmt=new char[strlen(name)+1];
			nameAnim=new char[strlen(name)+1];
			}
		catch(bad_alloc &)
			{
			printf("FAIL: Sprite.load: Nie udalo sie zajac miejsca na nazwy plikow\n");
			//if(nameGfx) delete [] nameGfx;
			if(nameSmt) delete [] nameSmt;
			if(nameAnim) delete [] nameAnim;
			}
		// Wpisywanie nazwy pliku z alfa
		strcpy(nameSmt, name);
		strcpy(nameSmt+strlen(name)-3, "smt");
		// Wpisywanie nazwy pliku z info o animacjach
		strcpy(nameAnim, name);
		strcpy(nameAnim+strlen(name)-3, "txt");
		//printf("INFO: Sprite.load: Plik %s/%s/%s\n", name, nameSmt, nameAnim);
		try
			{
			gfx[name]=new Sprite;													// Dodawanie do tablicy
			}
		catch(bad_alloc &)
			{
			printf("FAIL: Sprite.load: Nie udalo sie dodac sprite do mapy (%s)\n", name);
			continue;
			}
	/********** Wczytywanie obrazka **********/
		if(!Sprite::loadGfx(name, gfx[name]))
			{
			printf("FAIL: Sprite.load: Nie wczytano grafiki\n");
			continue;
			}
		//Sprite *sprite=gfx[name];											// Wskaznik na aktualna grafike, nie ma sensu uzywac ciagle mapy
	/********** Ustawianie alfy **********/
		if(!Sprite::loadAlpha(nameSmt, gfx[name]))
			{
			printf("FAIL: Sprite.load: Nie wczytano alfy\n");
			continue;
			}
		//printf("OK  : Sprite.load: Alfa ustawiona\n");
	/********** Tworzenie maski **********/
		if(!Sprite::loadMask(gfx[name]))
			{
			printf("FAIL: Sprite.load: Nie wygenerowano maski\n");
			continue;
			}
		//printf("OK  : Sprite.load: Maska stworzona\n");
	/********** Wczytywanie animacji **********/
		if(!Sprite::loadAnims(nameAnim, gfx[name]))
			{
			printf("FAIL: Sprite.load: Nie wczytano animacji\n");
			continue;
			}
		//printf("OK  : Sprite.load: Wczytano %s\n", name);
		delete [] nameAnim;
		delete [] nameSmt;
		}
	if(count>0)
		printf("OK  : Sprite.load: Grafiki wczytane\n");
	else
		printf("FAIL: Sprite.load: Nie udalo sie wczytac grafik\n");
	}

void Sprite::print(Uint32 anim, int frame, int x, int y)
	{
	//printcount++;															// Licznik ilosci wywolan printa. Zmienna zrobiona w platf02-v2.cpp
	if(anim>=anims.size())
		{
		printf("FAIL: Sprite.print: Brak animacji %d\n", anim);
		//anim=anims.size()-1;
		return;
		}
	SDL_Rect offset;
	//rect(&tempRect, x-anims[anim].spotx, y-anims[anim].spoty, min(anims[anim].w, screen->w-x-anims[anim].spotx), min(anims[anim].h-anims[anim].spoty, screen->h-x));
	rect(&offset, x-anims[anim].spotx, y-anims[anim].spoty, anims[anim].w, anims[anim].h);
	SDL_Rect clip;
	rect(&clip, anims[anim].x0+frame*anims[anim].w, anims[anim].y0, anims[anim].w, anims[anim].h);
	SDL_BlitSurface(sprite, &clip, screen, &offset);
	//SDL_FillRect(screen, &offset, 0xFF0000);
	//putPix(screen, x, y, 0xFF0000);
	/*putPix(screen, x-anims[anim].spotx, y-anims[anim].spoty, 0x00FF00);
	putPix(screen, x-anims[anim].spotx+anims[anim].w, y-anims[anim].spoty, 0x00FF00);
	putPix(screen, x-anims[anim].spotx+anims[anim].w, y-anims[anim].spoty+anims[anim].h, 0x00FF00);
	putPix(screen, x-anims[anim].spotx, y-anims[anim].spoty+anims[anim].h, 0x00FF00);*/
	}

void Sprite::clear(map<string, Sprite *> &gfx)
	{
	printf("INFO: Sprite.load: Czyszczenie mapy\n");
	try
		{
		for(map<string, Sprite *>::iterator it=gfx.begin(); it!=gfx.end(); it++)
			{
			//printf("INFO: Sprite.clear: Kasowanie %s, %p -> %p\n", it->first.c_str(), it->second, it->second->sprite);
			if((*it).second==NULL)
				{
				printf("FAIL: Sprite.clear: NULL (%s)\n", (*it).first.c_str());
				continue;
				}
			if((*it).second->mask!=NULL)
				{
				for(int y=0; y<(*it).second->sprite->h; y++)
					{
					if((*it).second->mask[y]!=NULL)
						delete [] (*it).second->mask[y];
					}
				delete [] (*it).second->mask;
				}
			//delete [] (*it).second->sprite;
			//if((*it).second && (*it).second->sprite)			// Wybija >:
			//	SDL_FreeSurface((*it).second->sprite);
			(*it).second->anims.clear();
			delete it->second;
			}
		gfx.clear();
		}
	catch(bad_alloc &)
		{
		printf("FAIL: Sprite.load: Nie udalo sie wyczyscic mapy\n");
		return;
		}
	printf("OK  : Sprite.load: Skasowano grafiki\n");
	}

bool Sprite::getCol(int x, int y)
	{
	if(x<0 || x>sprite->w || y<0 || y>sprite->h || mask==NULL || mask[x]==NULL)
		return 0;
	return mask[x][y];
	}

Anim* Sprite::getAnim(int anim)
	{
	if(anim<0 || anim>(int)anims.size())
		{
		printf("FAIL: Sprite.getAnim: Nie ma takiej animacji (%d)\n", anim);
		return NULL;
		}
	return &anims[anim];
	}

int Sprite::getAnimCount()
	{
	return anims.size();
	}
