/// \file sprite.h
/// \brief Klasy dotyczace grafiki

/// \struct Anim
/// \brief Przechowuje informacje o konkretnej animacji
struct Anim
	{
	int frames, x0, y0, w, h, spotx, spoty, speed, returnto;
	};

/** \class Sprite
 *  \brief Klasa przechowujaca grafike i animacje
 *
 *  Posiada wbudowana funkcje wyswietlajaca.
 */

class Sprite
	{
	private:
		//static Resfile res;
		vector <Anim> anims;
		SDL_Surface *sprite;
		bool **mask;

		static bool loadGfx(const char *name, Sprite *sprite);
		static bool loadAlpha(const char *name, Sprite *sprite);
		static bool loadMask(Sprite *sprite);
		static bool loadAnims(const char *name, Sprite *sprite);
	public:
		/* /// Wektor z informacjami o animacjach. Odprywatyzowany bo tak jest fajniej
		//vector <Anim> anims;
		/// Maska kolizji, tablica dwuwymiarowa */

		/// \brief Wczytywanie grafik z \a filename do mapy \a gfx (referencja)
		/// \param filename String z nazwa pliku do wczytania
		/// \param gfx Referencja na mape string-Sprite*
		/// \return Nic nie zwraca o;
		static void load(string filename, map <string, Sprite*> &gfx);
		/// \brief Czyszczenie mapy z animacjami
		/// \param gfx Referencja na mape z grafikami
		static void clear(map<string, Sprite *> &gfx);
		/// \brief Wyswietlenie
		/// \param anim Uint32, numer animacji. Jesli bedzie wiekszy od ilosci animacji to nie wyswietli nic.
		/// \param frame Numer klatki
		/// \param x Wsolrzedne na ekranie...?
		/// \param y Jw
		/// \return void
		void print(Uint32 anim, int frame, int x, int y);

		bool getCol(int x, int y);
		Anim* getAnim(int anim);
		int getAnimCount();
	};
